import requests
from bs4 import BeautifulSoup
import pickle
from department import Department

class University:
    _saveFile = "univer_save.pickle"

    def __init__(self):
        self.departments = []
        self.loadData()
        self.fillDepartments()
        self.saveData()

    def fillDepartments(self):
        if not self.departments:
            cafsLink = 'https://www.mstu.edu.ru/structure/kafs/'
            html = requests.get(cafsLink).text
            soup = BeautifulSoup(html, features="html.parser")
            lists = soup.findAll("ul", {"class": "anker"})
            # print(lists)
            links = []
            for ul in lists:
                hrefs = ul.findAll("a")
                for h in hrefs:
                    links.append(h['href'])
            print(links)
            baseUrl = 'https://www.mstu.edu.ru'
            for link in links:
                url = baseUrl + link
                self.addDepartment(url)

    def addDepartment(self, link):
        dep = Department(link)
        self.departments.append(dep)

    def getDepartment(self, name):
        for dep in self.departments:
            if name == dep.name:
                return dep
        return None

    def getDepartmentNames(self):
        names = []
        for dep in self.departments:
            names.append(dep.name)
        return names

    def searchEmployees(self, name):
        names = []
        for dep in self.departments:
            names.extend(dep.searchEmployees(name))
        return names

    def getAuthor(self, fullName):
        empls = []
        for dep in self.departments:
            empl = dep.findEmployee(fullName)
            if empl is not None:
                empls.append(empl)
        if len(empls) != 1:
            return None
        return empls[0]

    def loadData(self):
        try:
            with open(University._saveFile, 'rb') as f:
                copy = pickle.load(f)
                if copy is not None:
                    self.departments = copy.departments
        except EnvironmentError:
            pass

    def saveData(self):
        with open(University._saveFile, 'wb') as f:
            pickle.dump(self, f)




from transliterate import translit
from bs4 import BeautifulSoup


class Author:

    def __init__(self, name):
        self.name = name
        self.engName = translit(name, 'ru', reversed=True)
        self.crossrefName = []
        self.scopusName = []
        self.affiliations = []
        self.orcid = None
        self.publications = []

    def getScopusName(self):
        if not self.scopusName:
            splitName = self.engName.split(' ')
            self.scopusName = (splitName[0], splitName[1][0] + '.' + splitName[2][0] + '.')
        return self.scopusName




import university

_univer = None


def getUniversity():
    global _univer
    if _univer is None:
        _univer = university.University()
    return _univer


if __name__ == "__main__":
    getUniversity()


import requests
from bs4 import BeautifulSoup
from author import Author


class Department:

    def __init__(self, link):
        self.siteLink = link
        self.name = ""
        self.employees = []
        self.fillData()

    def fillData(self):
        r = requests.get(self.siteLink)
        soup = BeautifulSoup(r.text, features="html.parser")
        title = soup.find("h1")
        if title.text == "Ботанический сад": # не является кафедрой, другой интерфейс
            return

        self.name = title.text
        r = requests.get(self.siteLink + "prepods/")
        soup = BeautifulSoup(r.text, features="html.parser")
        table = soup.find("table")
        table_body = table.find('tbody')
        rows = table_body.find_all('tr')
        for row in rows:
            cols = row.find_all('td')
            name = cols[0].text
            author = Author(name)
            self.employees.append(author)

    def searchEmployees(self, name):
        names = []
        for empl in self.employees:
            if name.lower() in empl.name.lower():
                names.append(empl.name)
        return names

    def findEmployee(self, name):
        empls = []
        for empl in self.employees:
            if name.lower() in empl.name.lower():
                empls.append(empl)
        if len(empls) != 1:
            return None
        return empls[0]

    def createReport(self):
        publications = []

        from datetime import date
        from dateutil.relativedelta import relativedelta
        for empl in self.employees:
            if empl is not None:
                toDate = date.today()
                fromDate = toDate - relativedelta(years=1)

                #from API import crossref_api
                #publs = crossref_api.GetPublicationsByPeriod(empl, fromDate, toDate)
                #publications.extend(publs)
                #data - список публикаций сотрудника (каждая - в виде dict {title, author, createdDate}

        #from Reports.word_creator import createForm2
        #createForm2(publications)



import datetime
from prettytable import PrettyTable


class Publication:
    defaultDate = datetime.datetime(1970, 1, 1)

    def __init__(self):
        self.title = ""
        self.authors = []
        self.type = ""
        self.issn = None
        self.doi = None
        self.eid = None
        self.scopusId = None
        self.pii = None
        self.ut = None  # wos
        self.publishedDate = Publication.defaultDate
        self.indexedDate = Publication.defaultDate
        self.citations = 0
        self.publisher = ""
        self.containerTitle = []
        # self.pages = "" # для статьи в журнале

    def MergeIDs(self, anotherPubl):
        if anotherPubl is None:
            return

        if self.eid is None:
            self.eid = anotherPubl.eid
        if self.scopusId is None:
            self.scopusId = anotherPubl.scopusId
        if self.pii is None:
            self.pii = anotherPubl.pii
        if self.ut is None:
            self.ut = anotherPubl.ut

    def __str__(self):
        table = PrettyTable()
        table.header = False
        table._max_width = {"Field 1": 20, "Field 2": 70}

        table.add_row(["Название", self.title[0]])
        table.add_row(["ISSN", self.issn])
        table.add_row(["DOI", self.doi])
        table.add_row(["EID", self.eid])
        table.add_row(["Scopus ID", self.scopusId])
        table.add_row(["PII", self.pii])
        table.add_row(["Дата публикации", str(self.publishedDate)])
        table.add_row(["Дата индексирования", str(self.indexedDate)])
        table.add_row(["Число цитирований", self.citations])
        table.add_row(["Издатель", self.publisher])
        table.add_row(["Источник", self.containerTitle])
        return table.get_string()





import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="kvv_StoredObjects",
    version="0.0.4",
    author="Vladislav Kornilov",
    author_email="v.kornilovv@yandex.ru",
    description="StoredObjects is a helper package with University objects for the Publications Activity Module",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/VladKornilov/StoredObjects",
    project_urls={
        "Bug Tracker": "https://github.com/VladKornilov/StoredObjects/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
    install_requires=[
        'requests',
        'beautifulsoup4',
        'transliterate',
        'prettytable'
    ],
    python_requires=">=3.6",
)